@@ -1,5 +1,5 @@
 /* test-register.c --- Self tests for IDNA processing
-   Copyright (C) 2011-2016 Simon Josefsson
+   Copyright (C) 2011-2017 Simon Josefsson
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -27,8 +27,8 @@
 
 struct idna
 {
-  const uint8_t *alabel;
-  const uint8_t *ulabel;
+  const char *alabel;
+  const char *ulabel;
   const char *out;
   int rc;
   int flags;
@@ -92,11 +92,10 @@ static const struct idna idna[] = {
    "xn--foobarbaz-dl5hq7z", IDN2_OK},
 };
 
-int debug = 1;
-int error_count = 0;
-int break_on_error = 1;
+static int error_count = 0;
+static int break_on_error = 1;
 
-void
+_GL_ATTRIBUTE_FORMAT_PRINTF (1, 2) static void
 fail (const char *format, ...)
 {
   va_list arg_ptr;
@@ -109,29 +108,11 @@ fail (const char *format, ...)
     exit (EXIT_FAILURE);
 }
 
-void
-hexprint (const char *str, size_t len)
-{
-  size_t i;
-
-  printf ("\t;; ");
-  if (str && len)
-    for (i = 0; i < len; i++)
-      {
-	printf ("%02x ", (str[i] & 0xFF));
-	if ((i + 1) % 8 == 0)
-	  printf (" ");
-	if ((i + 1) % 16 == 0 && i + 1 < len)
-	  printf ("\n\t;; ");
-      }
-  printf ("\n");
-}
-
 int
 main (void)
 {
   uint8_t *out;
-  size_t i;
+  unsigned i;
   int rc;
 
   puts ("-----------------------------------------------------------"
@@ -143,33 +124,71 @@ main (void)
 	"-------------------------------------");
   for (i = 0; i < sizeof (idna) / sizeof (idna[0]); i++)
     {
-      rc = idn2_register_u8 (idna[i].ulabel, idna[i].alabel,
+      out = (uint8_t *) 0x1234;
+      rc = idn2_register_u8 ((uint8_t*)idna[i].ulabel, (uint8_t*)idna[i].alabel,
 			     &out, idna[i].flags);
-      printf ("%3d  %-25s %-25s %-25s %s\n", i, idn2_strerror_name (rc),
+      printf ("%3u  %-25s %-25s %-25s %s\n", i, idn2_strerror_name (rc),
 	      rc == IDN2_OK ? idna[i].out : "",
-	      idna[i].alabel ? (char *) idna[i].alabel : "(null)",
-	      idna[i].ulabel ? (char *) idna[i].ulabel : "(null)");
+	      idna[i].alabel ? idna[i].alabel : "(null)",
+	      idna[i].ulabel ? idna[i].ulabel : "(null)");
       if (rc != idna[i].rc)
 	fail ("expected rc %d got rc %d\n", idna[i].rc, rc);
-      else if (rc == IDN2_OK && strcmp (out, idna[i].out) != 0)
+      else if (rc == IDN2_OK && strcmp ((char*)out, idna[i].out) != 0)
 	fail ("expected: %s\ngot: %s\n", idna[i].out, out);
 
       if (rc == IDN2_OK)
 	{
 	  uint8_t *tmp;
 
-	  rc = idn2_lookup_u8 (idna[i].ulabel, &tmp, idna[i].flags);
+	  if (out == (void *) 0x1234)
+	    fail ("out has not been set");
+
+	  rc = idn2_lookup_u8 ((uint8_t*)idna[i].ulabel, &tmp, idna[i].flags);
 	  if (rc != IDN2_OK)
-	    fail ("lookup failed?! tv %d", i);
-	  if (strcmp (out, tmp) != 0)
+	    fail ("lookup failed?! tv %u", i);
+	  if (strcmp ((char*)out, (char*)tmp) != 0)
 	    fail ("lookup and register different? lookup %s register %s\n",
 		  tmp, out);
-	  free (tmp);
-	  free (out);
+	  idn2_free (tmp);
+	  idn2_free (out);
+	}
+      else
+	{
+	  if (out != (void *) 0x1234)
+	    fail ("out has been tainted on error");
 	}
     }
   puts ("-----------------------------------------------------------"
 	"-------------------------------------");
 
+  /* special calls to cover edge cases */
+  if ((rc = idn2_register_u8 (NULL, NULL, NULL, 0)) != IDN2_OK)
+    fail ("special #1 failed with %d\n", rc);
+
+  out = (uint8_t *) 0x123;
+  if ((rc = idn2_register_u8 (NULL, NULL, &out, 0)) != IDN2_OK)
+    fail ("special #2 failed with %d\n", rc);
+  if (out)
+    fail ("special #2 failed with out!=NULL\n");
+
+  if ((rc = idn2_register_u8 (NULL, (uint8_t*)"xn+-xxx", &out, 0)) != IDN2_INVALID_ALABEL)
+    fail ("special #3 failed with %d\n", rc);
+  if (out)
+    fail ("special #3 failed with out!=NULL\n");
+
+  if ((rc = idn2_register_u8 (NULL, (uint8_t*)"xn--\xff", &out, 0)) != IDN2_INVALID_ALABEL)
+    fail ("special #4 failed with %d\n", rc);
+  if (out)
+    fail ("special #4 failed with out!=NULL\n");
+
+  if ((rc = idn2_register_ul (NULL, NULL, NULL, 0)) != IDN2_OK)
+    fail ("special #5 failed with %d\n", rc);
+
+  if ((rc = idn2_register_ul ("foo", NULL, NULL, 0)) != IDN2_OK)
+    fail ("special #6 failed with %d\n", rc);
+
+  if ((rc = idn2_register_u8 ((uint8_t*)"faß", NULL, NULL, IDN2_NFC_INPUT)) != IDN2_OK)
+    fail ("special #7 failed with %d\n", rc);
+
   return error_count;
 }
